/*
 * Copyright 10/20/06 Sun Microsystems, Inc.  All Rights Reserved.
 */

/*
 * These macros are predefined by the C and C++ compilers when
 *      the -xprefetch=explicit option, or equivalent, has been specified
 *
 *   sparc_prefetch_read_once( addr )
 *   sun_prefetch_read_once( addr )
 *      prefetch data for one read
 *          addr must be an expression for the address to be prefetched
 *
 *   sparc_prefetch_read_many( addr )
 *   sun_prefetch_read_many( addr )
 *      prefetch data for many reads
 *          addr must be an expression for the address to be prefetched
 *
 *   sparc_prefetch_write_once( addr )
 *   sun_prefetch_write_once( addr )
 *      prefetch data for write once
 *          addr must be an expression for the address to be prefetched
 *
 *   sparc_prefetch_write_many( addr )
 *   sun_prefetch_write_many( addr )
 *      prefetch data for many writes
 *          addr must be an expression for the address to be prefetched
 *
 */

#ifndef _SUN_PREFETCH_H
#define _SUN_PREFETCH_H

#ifdef  __cplusplus
extern "C" {
#endif

#if defined(__sparc) || defined(__sparcv9)

extern void __sparc_prefetch_read_once_intrinsic( void *address );
extern void __sparc_prefetch_read_many_intrinsic( void *address );
extern void __sparc_prefetch_write_once_intrinsic( void *address );
extern void __sparc_prefetch_write_many_intrinsic( void *address );
extern void __sparc_prefetch_instruction_intrinsic( void *address );
extern void __sparc_strong_prefetch_read_once_intrinsic( void *address );
extern void __sparc_strong_prefetch_read_many_intrinsic( void *address );
extern void __sparc_strong_prefetch_write_once_intrinsic( void *address );
extern void __sparc_strong_prefetch_write_many_intrinsic( void *address );
#pragma no_side_effect ( __sparc_prefetch_read_once_intrinsic )
#pragma no_side_effect ( __sparc_prefetch_read_many_intrinsic )
#pragma no_side_effect ( __sparc_prefetch_write_once_intrinsic )
#pragma no_side_effect ( __sparc_prefetch_write_many_intrinsic )
#pragma no_side_effect ( __sparc_prefetch_instruction_intrinsic )
#pragma no_side_effect ( __sparc_strong_prefetch_read_once_intrinsic )
#pragma no_side_effect ( __sparc_strong_prefetch_read_many_intrinsic )
#pragma no_side_effect ( __sparc_strong_prefetch_write_once_intrinsic )
#pragma no_side_effect ( __sparc_strong_prefetch_write_many_intrinsic )

extern void __sparc_prefetch_aggressive_region_begin_intrinsic( void );
extern void __sparc_prefetch_aggressive_region_end_intrinsic( void );

#pragma no_side_effect( __sparc_prefetch_aggressive_region_begin_intrinsic )
#pragma no_side_effect( __sparc_prefetch_aggressive_region_end_intrinsic )

extern void __sparc_prefetch_moderate_region_begin_intrinsic( void );
extern void __sparc_prefetch_moderate_region_end_intrinsic( void );

#pragma no_side_effect( __sparc_prefetch_moderate_region_begin_intrinsic )
#pragma no_side_effect( __sparc_prefetch_moderate_region_end_intrinsic )

extern void __sparc_prefetch_conservative_region_begin_intrinsic( void );
extern void __sparc_prefetch_conservative_region_end_intrinsic( void );

#pragma no_side_effect( __sparc_prefetch_conservative_region_begin_intrinsic )
#pragma no_side_effect( __sparc_prefetch_conservative_region_end_intrinsic )

extern void __sparc_prefetch_none_region_begin_intrinsic( void );
extern void __sparc_prefetch_none_region_end_intrinsic( void );

#pragma no_side_effect( __sparc_prefetch_none_region_begin_intrinsic )
#pragma no_side_effect( __sparc_prefetch_none_region_end_intrinsic )

/* generic prefetch pragmas for sparc and x86 */
#define sun_prefetch_read_once(addr) \
		sparc_prefetch_read_once( addr )
#define sun_prefetch_read_many(addr) \
		sparc_prefetch_read_many( addr )
#define sun_prefetch_write_once(addr) \
		sparc_prefetch_write_once( addr )
#define sun_prefetch_write_many(addr) \
		sparc_prefetch_write_many( addr )

#ifdef __SUN_PREFETCH

#define sparc_prefetch_read_once(addr) \
		__sparc_prefetch_read_once_intrinsic( addr )
#define sparc_prefetch_read_many(addr) \
		__sparc_prefetch_read_many_intrinsic( addr )
#define sparc_prefetch_write_once(addr) \
		__sparc_prefetch_write_once_intrinsic( addr )
#define sparc_prefetch_write_many(addr) \
		__sparc_prefetch_write_many_intrinsic( addr )
#define sparc_prefetch_instruction(addr) \
		__sparc_prefetch_instruction_intrinsic( addr )
#define sparc_strong_prefetch_read_once(addr) \
		__sparc_strong_prefetch_read_once_intrinsic( addr )
#define sparc_strong_prefetch_read_many(addr) \
		__sparc_strong_prefetch_read_many_intrinsic( addr )
#define sparc_strong_prefetch_write_once(addr) \
		__sparc_strong_prefetch_write_once_intrinsic( addr )
#define sparc_strong_prefetch_write_many(addr) \
		__sparc_strong_prefetch_write_many_intrinsic( addr )


#define __sparc_prefetch_aggressive_region_begin() \
		__sparc_prefetch_aggressive_region_begin_intrinsic()
#define __sparc_prefetch_aggressive_region_end() \
		__sparc_prefetch_aggressive_region_end_intrinsic()
#define __sparc_prefetch_moderate_region_begin() \
		__sparc_prefetch_moderate_region_begin_intrinsic()
#define __sparc_prefetch_moderate_region_end() \
		__sparc_prefetch_moderate_region_end_intrinsic()
#define __sparc_prefetch_conservative_region_begin() \
		__sparc_prefetch_conservative_region_begin_intrinsic()
#define __sparc_prefetch_conservative_region_end() \
		__sparc_prefetch_conservative_region_end_intrinsic()
#define __sparc_prefetch_none_region_begin() \
		__sparc_prefetch_none_region_begin_intrinsic()
#define __sparc_prefetch_none_region_end() \
		__sparc_prefetch_none_region_end_intrinsic()


#else   /* ! __SUN_PREFETCH */

#define sparc_prefetch_read_once(addr)
#define sparc_prefetch_read_many(addr)
#define sparc_prefetch_write_once(addr)
#define sparc_prefetch_write_many(addr)
#define sparc_prefetch_instruction(addr)
#define sparc_strong_prefetch_read_once(addr) 
#define sparc_strong_prefetch_read_many(addr)
#define sparc_strong_prefetch_write_once(addr) 
#define sparc_strong_prefetch_write_many(addr)

#define __sparc_prefetch_aggressive_region_begin()
#define __sparc_prefetch_aggressive_region_end()
#define __sparc_prefetch_moderate_region_begin()
#define __sparc_prefetch_moderate_region_end()
#define __sparc_prefetch_conservative_region_begin()
#define __sparc_prefetch_conservative_region_end()
#define __sparc_prefetch_none_region_begin()
#define __sparc_prefetch_none_region_end()

#endif  /* __SUN_PREFETCH */

#elif defined(__i386) || defined(__amd64) || defined(__x86_64)

extern void __sun_x86_prefetch_read_once_intrinsic( void *address );
extern void __sun_x86_prefetch_read_many_intrinsic( void *address );
extern void __sun_x86_prefetch_write_once_intrinsic( void *address );
extern void __sun_x86_prefetch_write_many_intrinsic( void *address );

#pragma no_side_effect ( __sun_x86_prefetch_read_once_intrinsic )
#pragma no_side_effect ( __sun_x86_prefetch_read_many_intrinsic )
#pragma no_side_effect ( __sun_x86_prefetch_write_once_intrinsic )
#pragma no_side_effect ( __sun_x86_prefetch_write_many_intrinsic )

#ifdef __SUN_PREFETCH  /* note, #defined by driver for -xprefetch=explicit */

/* generic prefetch pragmas for sparc and x86 */
#define sun_prefetch_read_once(addr) \
		__sun_x86_prefetch_read_once_intrinsic( addr )
#define sun_prefetch_read_many(addr) \
		__sun_x86_prefetch_read_many_intrinsic( addr )
#define sun_prefetch_write_once(addr) \
		__sun_x86_prefetch_write_once_intrinsic( addr )
#define sun_prefetch_write_many(addr) \
		__sun_x86_prefetch_write_many_intrinsic( addr )

#else   /* ! __SUN_PREFETCH */

#define sun_prefetch_read_once(addr)
#define sun_prefetch_read_many(addr)
#define sun_prefetch_write_once(addr)
#define sun_prefetch_write_many(addr)

#endif /* __SUN_PREFETCH */

#else
#error "neither __sparc nor __i386 is #defined"
#endif

#ifdef  __cplusplus
}
#endif

#endif  /* _SUN_PREFETCH_H */

