/*
 * Copyright 04/06/06 Sun Microsystems, Inc.  All Rights Reserved.
 */

#ifndef _SUNIMATH_H
#define	_SUNIMATH_H

#ifdef __cplusplus
extern "C" {
#endif

#pragma ident "@(#)sunimath.h 1.13 06/04/06 SMI"

#ifdef __P
#undef __P
#endif

#ifdef __STDC__
#define	__P(p)	p
#else
#define	__P(p)	()
#endif

/* definition of interval types */
typedef struct {
	double	l, u;
} d_interval;
typedef struct {
	float	l, u;
} r_interval;
typedef struct {
	long double	l, u;
} q_interval;

/*
 * d_interval interval elementary functions
 */

extern void	__di_acos_  __P((d_interval*,const d_interval*));
extern void 	__di_asin_  __P((d_interval*,const d_interval*));
extern void 	__di_atan_  __P((d_interval*,const d_interval*));
extern void 	__di_atan2_ __P((d_interval*,const d_interval*,const d_interval*));
extern void 	__di_cbrt_  __P((d_interval*,const d_interval*));
extern void 	__di_cos_   __P((d_interval*,const d_interval*));
extern void 	__di_cosh_  __P((d_interval*,const d_interval*));
extern void 	__di_exp_   __P((d_interval*,const d_interval*));
extern void 	__di_fmod_  __P((d_interval*,const d_interval*,const d_interval*));
extern void 	__di_log_   __P((d_interval*,const d_interval*));
extern void 	__di_log10_ __P((d_interval*,const d_interval*));
extern void	__di_sign_  __P((d_interval*,const d_interval*,const d_interval*));
extern void 	__di_sin_   __P((d_interval*,const d_interval*));
extern void 	__di_sinh_  __P((d_interval*,const d_interval*));
extern void 	__di_sqr_   __P((d_interval*,const d_interval*));
extern void 	__di_sqrt_  __P((d_interval*,const d_interval*));
extern void 	__di_tan_   __P((d_interval*,const d_interval*));
extern void 	__di_tanh_  __P((d_interval*,const d_interval*));

/*
 * q_interval interval elementary functions
 */

extern void	__qi_fmod_  __P((q_interval*,const q_interval*,const q_interval*));
extern void	__qi_sqrt_  __P((q_interval*,const q_interval*));
extern void	__qi_sign_  __P((q_interval*,const q_interval*,const q_interval*));
extern void	__qi_sqr_   __P((q_interval*,const q_interval*));

/*
 * r_interval interval elementary functions
 */

extern void	__ri_acos_  __P((r_interval*,const r_interval*));
extern void	__ri_asin_  __P((r_interval*,const r_interval*));
extern void	__ri_atan_  __P((r_interval*,const r_interval*));
extern void	__ri_atan2_ __P((r_interval*,const r_interval*,const r_interval*));
extern void 	__ri_cbrt_  __P((r_interval*,const r_interval*));
extern void	__ri_cos_   __P((r_interval*,const r_interval*));
extern void	__ri_cosh_  __P((r_interval*,const r_interval*));
extern void	__ri_exp_   __P((r_interval*,const r_interval*));
extern void	__ri_fmod_  __P((r_interval*,const r_interval*,const r_interval*));
extern void	__ri_log_   __P((r_interval*,const r_interval*));
extern void	__ri_log10_ __P((r_interval*,const r_interval*));
extern void	__ri_sign_  __P((r_interval*,const r_interval*,const r_interval*));
extern void	__ri_sin_   __P((r_interval*,const r_interval*));
extern void	__ri_sinh_  __P((r_interval*,const r_interval*));
extern void 	__ri_sqr_   __P((r_interval*,const r_interval*));
extern void	__ri_sqrt_  __P((r_interval*,const r_interval*));
extern void	__ri_tan_   __P((r_interval*,const r_interval*));
extern void	__ri_tanh_  __P((r_interval*,const r_interval*));

/* Interval abs */
extern void	__di_fabs_ __P((d_interval*,const d_interval*));
extern void	__ri_fabs_ __P((r_interval*,const r_interval*));
extern void	__qi_fabs_ __P((q_interval*,const q_interval*));

/*
 * Implementation of interval operations & interval specific functions
 */

/* Interval  Addition */
extern void	__ri_add_ __P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_add_ __P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_add_ __P((q_interval*,const q_interval*,const q_interval*));

/* Interval Subtraction */
extern void	__ri_sub_ __P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_sub_ __P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_sub_ __P((q_interval*,const q_interval*,const q_interval*));
extern void	__ri_dsub_ __P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_dsub_ __P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_dsub_ __P((q_interval*,const q_interval*,const q_interval*));

/* Interval Multiplication */
extern void	__ri_mul_ __P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_mul_ __P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_mul_ __P((q_interval*,const q_interval*,const q_interval*));
extern void	__ri_mul_nornd_ __P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_mul_nornd_ __P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_mul_nornd_ __P((q_interval*,const q_interval*,const q_interval*));
extern void	__ri_dmul_ __P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_dmul_ __P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_dmul_ __P((q_interval*,const q_interval*,const q_interval*));

/* Interval Division */
extern void	__ri_div_ __P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_div_ __P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_div_ __P((q_interval*,const q_interval*,const q_interval*));
extern void	__ri_div_nornd_ __P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_div_nornd_ __P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_div_nornd_ __P((q_interval*,const q_interval*,const q_interval*));
extern void	__ri_ddiv_ __P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_ddiv_ __P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_ddiv_ __P((q_interval*,const q_interval*,const q_interval*));
extern void	__ri_divx_ __P((r_interval*,const r_interval*,const r_interval*,const r_interval*));
extern void	__di_divx_ __P((d_interval*,const d_interval*,const d_interval*,const d_interval*));
extern void	__qi_divx_ __P((q_interval*,const q_interval*,const q_interval*,const q_interval*));

/* Interval Negation */
extern void	__ri_neg_ __P((r_interval*,const r_interval*));
extern void	__di_neg_ __P((d_interval*,const d_interval*));
extern void	__qi_neg_ __P((q_interval*,const q_interval*));

/* Infimim, supremum */
extern float 		__ri_inf_ __P((const r_interval*));
extern double		__di_inf_ __P((const d_interval*));
extern long double	__qi_inf_ __P((const q_interval*));
extern float		__ri_sup_ __P((const r_interval*));
extern double		__di_sup_ __P((const d_interval*));
extern long double	__qi_sup_ __P((const q_interval*));

/* Test for an empty interval */
extern int	__ri_isempty_ __P((const r_interval*));
extern int	__di_isempty_ __P((const d_interval*));
extern int 	__qi_isempty_ __P((const q_interval*));

/* Interval intersection */
extern void	__ri_ix_ __P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_ix_ __P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_ix_ __P((q_interval*,const q_interval*,const q_interval*));

/* Interval hull */
extern void	__ri_ih_ __P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_ih_ __P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_ih_ __P((q_interval*,const q_interval*,const q_interval*));

/* Test for  disjoint intervals */
extern int	__ri_dj_ __P((const r_interval*,const r_interval*));
extern int	__di_dj_ __P((const d_interval*,const d_interval*));
extern int	__qi_dj_ __P((const q_interval*,const q_interval*));

/* "In interior" relation */
extern int	__ri_int_ __P((const r_interval*,const r_interval*));
extern int	__di_int_ __P((const d_interval*,const d_interval*));
extern int	__qi_int_ __P((const q_interval*,const q_interval*));

/* "In" relation */
extern int	__i_in_ri_ 	__P((int, 		const r_interval*));
extern int	__xl_in_ri_ 	__P((long long, 	const r_interval*));
extern int	__r_in_ri_ 	__P((float, 		const r_interval*));
extern int	__d_in_ri_ 	__P((double, 		const r_interval*));
extern int	__q_in_ri_ 	__P((long double, 	const r_interval*));
extern int	__i_in_di_ 	__P((int, 		const d_interval*));
extern int	__xl_in_di_ 	__P((long long, 	const d_interval*));
extern int	__r_in_di_ 	__P((float, 		const d_interval*));
extern int	__d_in_di_ 	__P((double, 		const d_interval*));
extern int	__q_in_di_ 	__P((long double, 	const d_interval*));
extern int	__i_in_qi_ 	__P((int, 		const q_interval*));
extern int	__xl_in_qi_ 	__P((long long, 	const q_interval*));
extern int	__r_in_qi_ 	__P((float, 		const q_interval*));
extern int	__d_in_qi_ 	__P((double, 		const q_interval*));
extern int	__q_in_qi_ 	__P((long double, 	const q_interval*));

/* Subset, Superset and Proper Subset, Proper Superset */
extern int	__ri_sp_ __P((const r_interval*,const r_interval*));
extern int	__di_sp_ __P((const d_interval*,const d_interval*));
extern int	__qi_sp_ __P((const q_interval*,const q_interval*));
extern int	__ri_psp_ __P((const r_interval*,const r_interval*));
extern int	__di_psp_ __P((const d_interval*,const d_interval*));
extern int	__qi_psp_ __P((const q_interval*,const q_interval*));
extern int	__ri_sb_ __P((const r_interval*,const r_interval*));
extern int	__di_sb_ __P((const d_interval*,const d_interval*));
extern int	__qi_sb_ __P((const q_interval*,const q_interval*));
extern int	__ri_psb_ __P((const r_interval*,const r_interval*));
extern int	__di_psb_ __P((const d_interval*,const d_interval*));
extern int	__qi_psb_ __P((const q_interval*,const q_interval*));

/* Set relations */
extern int	__ri_seq_ __P((const r_interval*,const r_interval*));
extern int	__di_seq_ __P((const d_interval*,const d_interval*));
extern int	__qi_seq_ __P((const q_interval*,const q_interval*));
extern int	__ri_sne_ __P((const r_interval*,const r_interval*));
extern int	__di_sne_ __P((const d_interval*,const d_interval*));
extern int	__qi_sne_ __P((const q_interval*,const q_interval*));
extern int	__ri_slt_ __P((const r_interval*,const r_interval*));
extern int	__di_slt_ __P((const d_interval*,const d_interval*));
extern int	__qi_slt_ __P((const q_interval*,const q_interval*));
extern int	__ri_sle_ __P((const r_interval*,const r_interval*));
extern int	__di_sle_ __P((const d_interval*,const d_interval*));
extern int	__qi_sle_ __P((const q_interval*,const q_interval*));
extern int	__ri_sgt_ __P((const r_interval*,const r_interval*));
extern int	__di_sgt_ __P((const d_interval*,const d_interval*));
extern int	__qi_sgt_ __P((const q_interval*,const q_interval*));
extern int	__ri_sge_ __P((const r_interval*,const r_interval*));
extern int	__di_sge_ __P((const d_interval*,const d_interval*));
extern int	__qi_sge_ __P((const q_interval*,const q_interval*));

/* Certainly relations */
extern int	__ri_clt_ __P((const r_interval*,const r_interval*));
extern int	__di_clt_ __P((const d_interval*,const d_interval*));
extern int	__qi_clt_ __P((const q_interval*,const q_interval*));
extern int	__ri_cle_ __P((const r_interval*,const r_interval*));
extern int	__di_cle_ __P((const d_interval*,const d_interval*));
extern int	__qi_cle_ __P((const q_interval*,const q_interval*));
extern int	__ri_cgt_ __P((const r_interval*,const r_interval*));
extern int	__di_cgt_ __P((const d_interval*,const d_interval*));
extern int	__qi_cgt_ __P((const q_interval*,const q_interval*));
extern int	__ri_cge_ __P((const r_interval*,const r_interval*));
extern int	__di_cge_ __P((const d_interval*,const d_interval*));
extern int	__qi_cge_ __P((const q_interval*,const q_interval*));
extern int	__ri_ceq_ __P((const r_interval*,const r_interval*));
extern int	__di_ceq_ __P((const d_interval*,const d_interval*));
extern int	__qi_ceq_ __P((const q_interval*,const q_interval*));
extern int	__ri_cne_ __P((const r_interval*,const r_interval*));
extern int	__di_cne_ __P((const d_interval*,const d_interval*));
extern int	__qi_cne_ __P((const q_interval*,const q_interval*));

/* Possibly relations */
extern int	__ri_plt_ __P((const r_interval*,const r_interval*));
extern int	__di_plt_ __P((const d_interval*,const d_interval*));
extern int	__qi_plt_ __P((const q_interval*,const q_interval*));
extern int	__ri_ple_ __P((const r_interval*,const r_interval*));
extern int	__di_ple_ __P((const d_interval*,const d_interval*));
extern int	__qi_ple_ __P((const q_interval*,const q_interval*));
extern int	__ri_pgt_ __P((const r_interval*,const r_interval*));
extern int	__di_pgt_ __P((const d_interval*,const d_interval*));
extern int	__qi_pgt_ __P((const q_interval*,const q_interval*));
extern int	__ri_pge_ __P((const r_interval*,const r_interval*));
extern int	__di_pge_ __P((const d_interval*,const d_interval*));
extern int	__qi_pge_ __P((const q_interval*,const q_interval*));
extern int	__ri_peq_ __P((const r_interval*,const r_interval*));
extern int	__di_peq_ __P((const d_interval*,const d_interval*));
extern int	__qi_peq_ __P((const q_interval*,const q_interval*));
extern int	__ri_pne_ __P((const r_interval*,const r_interval*));
extern int	__di_pne_ __P((const d_interval*,const d_interval*));
extern int	__qi_pne_ __P((const q_interval*,const q_interval*));

/* Min, Max */
extern void	__ri_max_ __P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_max_ __P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_max_ __P((q_interval*,const q_interval*,const q_interval*));
extern void	__ri_min_ __P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_min_ __P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_min_ __P((q_interval*,const q_interval*,const q_interval*));

/* Special interval functions */
extern float		__ri_mig_ __P((const r_interval*));
extern double		__di_mig_ __P((const d_interval*));
extern long double	__qi_mig_ __P((const q_interval*));
extern float		__ri_mag_ __P((const r_interval*));
extern double		__di_mag_ __P((const d_interval*));
extern long double	__qi_mag_ __P((const q_interval*));
extern float		__ri_mid_ __P((const r_interval*));
extern double		__di_mid_ __P((const d_interval*));
extern long double	__qi_mid_ __P((const q_interval*));
extern float		__ri_wid_ __P((const r_interval*));
extern double		__di_wid_ __P((const d_interval*));
extern long double	__qi_wid_ __P((const q_interval*));

/* Interval Random */
extern void	__ri_random_ 	__P((r_interval*));
extern void	__di_random_ 	__P((d_interval*));
extern void	__qi_random_ 	__P((q_interval*));

/* Ceiling,   Floor,   Int,   Aint,   Anint */
extern int		__ri_ceil_	__P((const r_interval *));
extern long long	__di_ceil_ 	__P((const d_interval *));
extern long long	__qi_ceil_ 	__P((const q_interval *));
extern int		__ri_floor_	__P((const r_interval *));
extern long long	__di_floor_	__P((const d_interval *));
extern long long	__qi_floor_	__P((const q_interval *));
extern void		__ri_aint_ 	__P((r_interval*,const r_interval*));
extern void		__di_aint_ 	__P((d_interval*,const d_interval*));
extern void 		__qi_aint_ 	__P((q_interval*,const q_interval*));
extern void		__ri_anint_ 	__P((r_interval*,const r_interval*));
extern void		__di_anint_ 	__P((d_interval*,const d_interval*));
extern void		__qi_anint_ 	__P((q_interval*,const q_interval*));
extern int		__ri_int_i_ 	__P((const r_interval*));
extern int		__di_int_i_	__P((const d_interval*));
extern int		__qi_int_i_ 	__P((const q_interval*));
extern long long	__ri_int_xl_ 	__P((const r_interval*));
extern long long	__di_int_xl_ 	__P((const d_interval*));
extern long long	__qi_int_xl_ 	__P((const q_interval*));

/* Ndigits */
extern int	__ri_ndigits_ __P((const r_interval*));
extern int	__di_ndigits_ __P((const d_interval*));
extern int	__qi_ndigits_ __P((const q_interval*));

/* Pow */
extern void	__ri_pow_i_ 	__P((r_interval*,const r_interval*,const int*));
extern void	__ri_pow_xl_ 	__P((r_interval*,const r_interval*,const long long*));
extern void	__di_pow_i_ 	__P((d_interval*,const d_interval*,const int*));
extern void	__di_pow_xl_ 	__P((d_interval*,const d_interval*,const long long*));
extern void	__qi_pow_xl_ 	__P((q_interval*,const q_interval*,const long long*));
extern void	__qi_pow_i_ 	__P((q_interval*,const q_interval*,const int*));
extern void	__ri_pow_ 	__P((r_interval*,const r_interval*,const r_interval*));
extern void	__di_pow_ 	__P((d_interval*,const d_interval*,const d_interval*));
extern void	__qi_pow_ 	__P((q_interval*,const q_interval*,const q_interval*));
extern void	__ri_pow_safe_i_ 	__P((r_interval*,const r_interval*,const d_interval*));
extern void	__ri_pow_safe_xl_ 	__P((r_interval*,const r_interval*,const q_interval*));
extern void	__di_pow_safe_i_ 	__P((d_interval*,const d_interval*,const d_interval*));
extern void	__di_pow_safe_xl_ 	__P((d_interval*,const d_interval*,const q_interval*));
extern void	__qi_pow_safe_i_ 	__P((q_interval*,const q_interval*,const d_interval*));
extern void	__qi_pow_safe_xl_ 	__P((q_interval*,const q_interval*,const q_interval*));

/* Interval constructor */
extern void	__ri_interval_i_ 	__P((r_interval*, int));
extern void	__ri_interval_xl_	__P((r_interval*, long long));
extern void	__ri_interval_r_ 	__P((r_interval*, float));
extern void	__ri_interval_d_ 	__P((r_interval*, double));
extern void	__ri_interval_q_ 	__P((r_interval*, long double));
extern void	__ri_interval_ri_ 	__P((r_interval*,const r_interval*));
extern void	__ri_interval_qi_ 	__P((r_interval*,const q_interval*));
extern void	__ri_interval_di_ 	__P((r_interval*,const d_interval*));
extern void	__ri_interval_i_i_ 	__P((r_interval*, int, int));
extern void	__ri_interval_xl_xl_ 	__P((r_interval*, long long,   long long));
extern void	__ri_interval_r_r_ 	__P((r_interval*, float,   float));
extern void	__ri_interval_d_d_ 	__P((r_interval*, double,   double));
extern void	__ri_interval_q_q_ 	__P((r_interval*, long double,   long double));
extern void	__di_interval_i_ 	__P((d_interval*, int));
extern void	__di_interval_xl_ 	__P((d_interval*, long long));
extern void	__di_interval_r_ 	__P((d_interval*, float));
extern void	__di_interval_d_ 	__P((d_interval*, double));
extern void	__di_interval_q_ 	__P((d_interval*, long double));
extern void	__di_interval_di_ 	__P((d_interval*,const d_interval*));
extern void	__di_interval_qi_ 	__P((d_interval*,const q_interval*));
extern void	__di_interval_ri_ 	__P((d_interval*,const r_interval*));
extern void	__di_interval_i_i_ 	__P((d_interval*, int,   int));
extern void	__di_interval_xl_xl_ 	__P((d_interval*, long long,   long long));
extern void	__di_interval_r_r_ 	__P((d_interval*, float,   float));
extern void	__di_interval_d_d_ 	__P((d_interval*, double,   double));
extern void	__di_interval_q_q_ 	__P((d_interval*, long double,   long double));
extern void	__qi_interval_i_ 	__P((q_interval*, int));
extern void	__qi_interval_xl_ 	__P((q_interval*, long long));
extern void	__qi_interval_r_ 	__P((q_interval*, float));
extern void	__qi_interval_d_ 	__P((q_interval*, double));
extern void	__qi_interval_q_ 	__P((q_interval*, long double));
extern void	__qi_interval_qi_ 	__P((q_interval*,const q_interval*));
extern void	__qi_interval_di_ 	__P((q_interval*,const d_interval*));
extern void	__qi_interval_ri_ 	__P((q_interval*,const r_interval*));
extern void	__qi_interval_i_i_ 	__P((q_interval*, int,   int));
extern void	__qi_interval_xl_xl_ 	__P((q_interval*, long long,   long long));
extern void	__qi_interval_r_r_ 	__P((q_interval*, float,   float));
extern void	__qi_interval_d_d_ 	__P((q_interval*, double,   double));
extern void	__qi_interval_q_q_ 	__P((q_interval*, long double,   long double));

/*
 *	Arguments and results passed by value
 */
/*
 * d_interval interval elementary functions
 */

extern d_interval	__di_acos	__P((d_interval));
extern d_interval 	__di_asin	__P((d_interval));
extern d_interval 	__di_atan	__P((d_interval));
extern d_interval 	__di_atan2	__P((d_interval, d_interval));
extern d_interval 	__di_cbrt	__P((d_interval));
extern d_interval 	__di_cos	__P((d_interval));
extern d_interval 	__di_cosh	__P((d_interval));
extern d_interval 	__di_exp	__P((d_interval));
extern d_interval 	__di_fmod	__P((d_interval, d_interval));
extern d_interval 	__di_log	__P((d_interval));
extern d_interval 	__di_log10	__P((d_interval));
extern d_interval	__di_sign	__P((d_interval, d_interval));
extern d_interval 	__di_sin	__P((d_interval));
extern d_interval 	__di_sinh	__P((d_interval));
extern d_interval 	__di_sqr	__P((d_interval));
extern d_interval 	__di_sqrt	__P((d_interval));
extern d_interval 	__di_tan	__P((d_interval));
extern d_interval 	__di_tanh	__P((d_interval));

/*
 * q_interval interval elementary functions
 */

extern q_interval	__qi_fmod	__P((q_interval, q_interval));
extern q_interval	__qi_sqrt	__P((q_interval));
extern q_interval	__qi_sign	__P((q_interval, q_interval));
extern q_interval	__qi_sqr	__P((q_interval));

/*
 * r_interval interval elementary functions
 */

extern r_interval	__ri_acos	__P((r_interval));
extern r_interval	__ri_asin	__P((r_interval));
extern r_interval	__ri_atan	__P((r_interval));
extern r_interval	__ri_atan2	__P((r_interval, r_interval));
extern r_interval 	__ri_cbrt	__P((r_interval));
extern r_interval	__ri_cos	__P((r_interval));
extern r_interval	__ri_cosh	__P((r_interval));
extern r_interval	__ri_exp	__P((r_interval));
extern r_interval	__ri_fmod	__P((r_interval, r_interval));
extern r_interval	__ri_log	__P((r_interval));
extern r_interval	__ri_log10	__P((r_interval));
extern r_interval	__ri_sign	__P((r_interval, r_interval));
extern r_interval	__ri_sin	__P((r_interval));
extern r_interval	__ri_sinh	__P((r_interval));
extern r_interval 	__ri_sqr	__P((r_interval));
extern r_interval	__ri_sqrt	__P((r_interval));
extern r_interval	__ri_tan	__P((r_interval));
extern r_interval	__ri_tanh	__P((r_interval));

/* Interval abs */
extern d_interval	__di_fabs	__P((d_interval));
extern r_interval	__ri_fabs	__P((r_interval));
extern q_interval	__qi_fabs	__P((q_interval));

/*
 * Implementation of interval operations & interval specific functions
 */

/* Interval  Addition */
extern r_interval	__ri_add	__P((r_interval, r_interval));
extern d_interval	__di_add	__P((d_interval, d_interval));
extern q_interval	__qi_add	__P((q_interval, q_interval));

/* Interval Subtraction */
extern r_interval	__ri_sub	__P((r_interval, r_interval));
extern d_interval	__di_sub	__P((d_interval, d_interval));
extern q_interval	__qi_sub	__P((q_interval, q_interval));
extern r_interval	__ri_dsub	__P((r_interval, r_interval));
extern d_interval	__di_dsub	__P((d_interval, d_interval));
extern q_interval	__qi_dsub	__P((q_interval, q_interval));

/* Interval Multiplication */
extern r_interval	__ri_mul	__P((r_interval, r_interval));
extern d_interval	__di_mul	__P((d_interval, d_interval));
extern q_interval	__qi_mul	__P((q_interval, q_interval));
extern r_interval	__ri_mul_nornd	__P((r_interval, r_interval));
extern d_interval	__di_mul_nornd	__P((d_interval, d_interval));
extern q_interval	__qi_mul_nornd	__P((q_interval, q_interval));
extern r_interval	__ri_dmul	__P((r_interval, r_interval));
extern d_interval	__di_dmul	__P((d_interval, d_interval));
extern q_interval	__qi_dmul	__P((q_interval, q_interval));

/* Interval Division */
extern r_interval	__ri_div	__P((r_interval, r_interval));
extern d_interval	__di_div	__P((d_interval, d_interval));
extern q_interval	__qi_div	__P((q_interval, q_interval));
extern r_interval	__ri_div_nornd	__P((r_interval, r_interval));
extern d_interval	__di_div_nornd	__P((d_interval, d_interval));
extern q_interval	__qi_div_nornd	__P((q_interval, q_interval));
extern r_interval	__ri_ddiv	__P((r_interval, r_interval));
extern d_interval	__di_ddiv	__P((d_interval, d_interval));
extern q_interval	__qi_ddiv	__P((q_interval, q_interval));
extern r_interval	__ri_divx	__P((r_interval, r_interval, r_interval));
extern d_interval	__di_divx	__P((d_interval, d_interval, d_interval));
extern q_interval	__qi_divx	__P((q_interval, q_interval, q_interval));

/* Interval Negation */
extern r_interval	__ri_neg	__P((r_interval));
extern d_interval	__di_neg	__P((d_interval));
extern q_interval	__qi_neg	__P((q_interval));

/* Infimim, supremum */
extern float 		__ri_inf	__P((r_interval));
extern double		__di_inf	__P((d_interval));
extern long double	__qi_inf	__P((q_interval));
extern float		__ri_sup	__P((r_interval));
extern double		__di_sup	__P((d_interval));
extern long double	__qi_sup	__P((q_interval));

/* Test for an empty interval */
extern int	__ri_isempty	__P((r_interval));
extern int	__di_isempty	__P((d_interval));
extern int 	__qi_isempty	__P((q_interval));

/* Interval intersection */
extern r_interval	__ri_ix	__P((r_interval, r_interval));
extern d_interval	__di_ix	__P((d_interval, d_interval));
extern q_interval	__qi_ix	__P((q_interval, q_interval));

/* Interval hull */
extern r_interval	__ri_ih	__P((r_interval, r_interval));
extern d_interval	__di_ih	__P((d_interval, d_interval));
extern q_interval	__qi_ih	__P((q_interval, q_interval));

/* Test for  disjoint intervals */
extern int	__ri_dj	__P((r_interval, r_interval));
extern int	__di_dj	__P((d_interval, d_interval));
extern int	__qi_dj	__P((q_interval, q_interval));

/* "In interior" relation */
extern int	__ri_int	__P((r_interval, r_interval));
extern int	__di_int	__P((d_interval, d_interval));
extern int	__qi_int	__P((q_interval, q_interval));

/* "In" relation */
extern int	__i_in_ri	__P((int, 		r_interval));
extern int	__xl_in_ri	__P((long long, 	r_interval));
extern int	__r_in_ri	__P((float, 		r_interval));
extern int	__d_in_ri	__P((double, 		r_interval));
extern int	__q_in_ri	__P((long double, 	r_interval));
extern int	__i_in_di	__P((int, 		d_interval));
extern int	__xl_in_di	__P((long long, 	d_interval));
extern int	__r_in_di	__P((float, 		d_interval));
extern int	__d_in_di	__P((double, 		d_interval));
extern int	__q_in_di	__P((long double, 	d_interval));
extern int	__i_in_qi	__P((int, 		q_interval));
extern int	__xl_in_qi	__P((long long, 	q_interval));
extern int	__r_in_qi	__P((float, 		q_interval));
extern int	__d_in_qi	__P((double, 		q_interval));
extern int	__q_in_qi	__P((long double, 	q_interval));

/* Subset, Superset and Proper Subset, Proper Superset */
extern int	__ri_sp		__P((r_interval, r_interval));
extern int	__di_sp		__P((d_interval, d_interval));
extern int	__qi_sp		__P((q_interval, q_interval));
extern int	__ri_psp	__P((r_interval, r_interval));
extern int	__di_psp	__P((d_interval, d_interval));
extern int	__qi_psp	__P((q_interval, q_interval));
extern int	__ri_sb		__P((r_interval, r_interval));
extern int	__di_sb		__P((d_interval, d_interval));
extern int	__qi_sb		__P((q_interval, q_interval));
extern int	__ri_psb	__P((r_interval, r_interval));
extern int	__di_psb	__P((d_interval, d_interval));
extern int	__qi_psb	__P((q_interval, q_interval));

/* Set relations */
extern int	__ri_seq	__P((r_interval, r_interval));
extern int	__di_seq	__P((d_interval, d_interval));
extern int	__qi_seq	__P((q_interval, q_interval));
extern int	__ri_sne	__P((r_interval, r_interval));
extern int	__di_sne	__P((d_interval, d_interval));
extern int	__qi_sne	__P((q_interval, q_interval));
extern int	__ri_slt	__P((r_interval, r_interval));
extern int	__di_slt	__P((d_interval, d_interval));
extern int	__qi_slt	__P((q_interval, q_interval));
extern int	__ri_sle	__P((r_interval, r_interval));
extern int	__di_sle	__P((d_interval, d_interval));
extern int	__qi_sle	__P((q_interval, q_interval));
extern int	__ri_sgt	__P((r_interval, r_interval));
extern int	__di_sgt	__P((d_interval, d_interval));
extern int	__qi_sgt	__P((q_interval, q_interval));
extern int	__ri_sge	__P((r_interval, r_interval));
extern int	__di_sge	__P((d_interval, d_interval));
extern int	__qi_sge	__P((q_interval, q_interval));

/* Certainly relations */
extern int	__ri_clt	__P((r_interval, r_interval));
extern int	__di_clt	__P((d_interval, d_interval));
extern int	__qi_clt	__P((q_interval, q_interval));
extern int	__ri_cle	__P((r_interval, r_interval));
extern int	__di_cle	__P((d_interval, d_interval));
extern int	__qi_cle	__P((q_interval, q_interval));
extern int	__ri_cgt	__P((r_interval, r_interval));
extern int	__di_cgt	__P((d_interval, d_interval));
extern int	__qi_cgt	__P((q_interval, q_interval));
extern int	__ri_cge	__P((r_interval, r_interval));
extern int	__di_cge	__P((d_interval, d_interval));
extern int	__qi_cge	__P((q_interval, q_interval));
extern int	__ri_ceq	__P((r_interval, r_interval));
extern int	__di_ceq	__P((d_interval, d_interval));
extern int	__qi_ceq	__P((q_interval, q_interval));
extern int	__ri_cne	__P((r_interval, r_interval));
extern int	__di_cne	__P((d_interval, d_interval));
extern int	__qi_cne	__P((q_interval, q_interval));

/* Possibly relations */
extern int	__ri_plt	__P((r_interval, r_interval));
extern int	__di_plt	__P((d_interval, d_interval));
extern int	__qi_plt	__P((q_interval, q_interval));
extern int	__ri_ple	__P((r_interval, r_interval));
extern int	__di_ple	__P((d_interval, d_interval));
extern int	__qi_ple	__P((q_interval, q_interval));
extern int	__ri_pgt	__P((r_interval, r_interval));
extern int	__di_pgt	__P((d_interval, d_interval));
extern int	__qi_pgt	__P((q_interval, q_interval));
extern int	__ri_pge	__P((r_interval, r_interval));
extern int	__di_pge	__P((d_interval, d_interval));
extern int	__qi_pge	__P((q_interval, q_interval));
extern int	__ri_peq	__P((r_interval, r_interval));
extern int	__di_peq	__P((d_interval, d_interval));
extern int	__qi_peq	__P((q_interval, q_interval));
extern int	__ri_pne	__P((r_interval, r_interval));
extern int	__di_pne	__P((d_interval, d_interval));
extern int	__qi_pne	__P((q_interval, q_interval));

/* Min, Max */
extern r_interval	__ri_max	__P((r_interval, r_interval));
extern d_interval	__di_max	__P((d_interval, d_interval));
extern q_interval	__qi_max	__P((q_interval, q_interval));
extern r_interval	__ri_min	__P((r_interval, r_interval));
extern d_interval	__di_min	__P((d_interval, d_interval));
extern q_interval	__qi_min	__P((q_interval, q_interval));

/* Special interval functions */
extern float		__ri_mig	__P((r_interval));
extern double		__di_mig	__P((d_interval));
extern long double	__qi_mig	__P((q_interval));
extern float		__ri_mag	__P((r_interval));
extern double		__di_mag	__P((d_interval));
extern long double	__qi_mag	__P((q_interval));
extern float		__ri_mid	__P((r_interval));
extern double		__di_mid	__P((d_interval));
extern long double	__qi_mid	__P((q_interval));
extern float		__ri_wid	__P((r_interval));
extern double		__di_wid	__P((d_interval));
extern long double	__qi_wid	__P((q_interval));

/* Interval Random */
extern r_interval	__ri_random __P((void));
extern d_interval	__di_random __P((void));
extern q_interval	__qi_random __P((void));

/* Ceiling,   Floor,   Int,   Aint,   Anint */
extern int		__ri_ceil	__P((r_interval));
extern long long	__di_ceil	__P((d_interval));
extern long long	__qi_ceil	__P((q_interval));
extern int		__ri_floor	__P((r_interval));
extern long long	__di_floor	__P((d_interval));
extern long long	__qi_floor	__P((q_interval));
extern r_interval	__ri_aint	__P((r_interval));
extern d_interval	__di_aint	__P((d_interval));
extern q_interval 	__qi_aint	__P((q_interval));
extern r_interval	__ri_anint	__P((r_interval));
extern d_interval	__di_anint	__P((d_interval));
extern q_interval	__qi_anint	__P((q_interval));
extern int		__ri_int_i	__P((r_interval));
extern int		__di_int_i	__P((d_interval));
extern int		__qi_int_i	__P((q_interval));
extern long long	__ri_int_xl	__P((r_interval));
extern long long	__di_int_xl	__P((d_interval));
extern long long	__qi_int_xl	__P((q_interval));

/* Ndigits */
extern int	__ri_ndigits	__P((r_interval));
extern int	__di_ndigits	__P((d_interval));
extern int	__qi_ndigits	__P((q_interval));

/* Pow */
extern r_interval	__ri_pow_i	__P((r_interval, int));
extern r_interval	__ri_pow_xl	__P((r_interval, long long));
extern d_interval	__di_pow_i	__P((d_interval, int));
extern d_interval	__di_pow_xl	__P((d_interval, long long));
extern q_interval	__qi_pow_xl	__P((q_interval, long long));
extern q_interval	__qi_pow_i	__P((q_interval, int));
extern r_interval	__ri_pow	__P((r_interval, r_interval));
extern d_interval	__di_pow	__P((d_interval, d_interval));
extern q_interval	__qi_pow	__P((q_interval, q_interval));

/* Interval constructor */
extern r_interval	__ri_interval_i		__P((int));
extern r_interval	__ri_interval_xl	__P((long long));
extern r_interval	__ri_interval_r		__P((float));
extern r_interval	__ri_interval_d		__P((double));
extern r_interval	__ri_interval_q		__P((long double));
extern r_interval	__ri_interval_ri	__P((r_interval));
extern r_interval	__ri_interval_qi	__P((q_interval));
extern r_interval	__ri_interval_di	__P((d_interval));
extern r_interval	__ri_interval_i_i	__P((int, int));
extern r_interval	__ri_interval_xl_xl	__P((long long,   long long));
extern r_interval	__ri_interval_r_r	__P((float,   float));
extern r_interval	__ri_interval_d_d	__P((double,   double));
extern r_interval	__ri_interval_q_q	__P((long double,   long double));
extern d_interval	__di_interval_i		__P((int));
extern d_interval	__di_interval_xl	__P((long long));
extern d_interval	__di_interval_r		__P((float));
extern d_interval	__di_interval_d		__P((double));
extern d_interval	__di_interval_q		__P((long double));
extern d_interval	__di_interval_di	__P((d_interval));
extern d_interval	__di_interval_qi	__P((q_interval));
extern d_interval	__di_interval_ri	__P((r_interval));
extern d_interval	__di_interval_i_i	__P((int,   int));
extern d_interval	__di_interval_xl_xl	__P((long long,   long long));
extern d_interval	__di_interval_r_r	__P((float,   float));
extern d_interval	__di_interval_d_d	__P((double,   double));
extern d_interval	__di_interval_q_q	__P((long double,   long double));
extern q_interval	__qi_interval_i		__P((int));
extern q_interval	__qi_interval_xl	__P((long long));
extern q_interval	__qi_interval_r		__P((float));
extern q_interval	__qi_interval_d		__P((double));
extern q_interval	__qi_interval_q		__P((long double));
extern q_interval	__qi_interval_qi	__P((q_interval));
extern q_interval	__qi_interval_di	__P((d_interval));
extern q_interval	__qi_interval_ri	__P((r_interval));
extern q_interval	__qi_interval_i_i	__P((int,   int));
extern q_interval	__qi_interval_xl_xl	__P((long long,   long long));
extern q_interval	__qi_interval_r_r	__P((float,   float));
extern q_interval	__qi_interval_d_d	__P((double,   double));
extern q_interval	__qi_interval_q_q	__P((long double,   long double));

/*
 *	Arguments passed by pointer
 *	Results passed by value
 */
/*
 * d_interval interval elementary functions
 */

extern d_interval	__di_acos_f  __P((const d_interval*));
extern d_interval 	__di_asin_f  __P((const d_interval*));
extern d_interval 	__di_atan_f  __P((const d_interval*));
extern d_interval 	__di_atan2_f __P((const d_interval*,const d_interval*));
extern d_interval 	__di_cbrt_f  __P((const d_interval*));
extern d_interval 	__di_cos_f   __P((const d_interval*));
extern d_interval 	__di_cosh_f  __P((const d_interval*));
extern d_interval 	__di_exp_f   __P((const d_interval*));
extern d_interval 	__di_fmod_f  __P((const d_interval*,const d_interval*));
extern d_interval 	__di_log_f   __P((const d_interval*));
extern d_interval 	__di_log10_f __P((const d_interval*));
extern d_interval	__di_sign_f  __P((const d_interval*,const d_interval*));
extern d_interval 	__di_sin_f   __P((const d_interval*));
extern d_interval 	__di_sinh_f  __P((const d_interval*));
extern d_interval 	__di_sqr_f   __P((const d_interval*));
extern d_interval 	__di_sqrt_f  __P((const d_interval*));
extern d_interval 	__di_tan_f   __P((const d_interval*));
extern d_interval 	__di_tanh_f  __P((const d_interval*));

/*
 * q_interval interval elementary functions
 */

extern q_interval	__qi_fmod_f  __P((const q_interval*,const q_interval*));
extern q_interval	__qi_sqrt_f  __P((const q_interval*));
extern q_interval	__qi_sign_f  __P((const q_interval*,const q_interval*));
extern q_interval	__qi_sqr_f   __P((const q_interval*));

/*
 * r_interval interval elementary functions
 */

extern r_interval	__ri_acos_f  __P((const r_interval*));
extern r_interval	__ri_asin_f  __P((const r_interval*));
extern r_interval	__ri_atan_f  __P((const r_interval*));
extern r_interval	__ri_atan2_f __P((const r_interval*,const r_interval*));
extern r_interval 	__ri_cbrt_f  __P((const r_interval*));
extern r_interval	__ri_cos_f   __P((const r_interval*));
extern r_interval	__ri_cosh_f  __P((const r_interval*));
extern r_interval	__ri_exp_f   __P((const r_interval*));
extern r_interval	__ri_fmod_f  __P((const r_interval*,const r_interval*));
extern r_interval	__ri_log_f   __P((const r_interval*));
extern r_interval	__ri_log10_f __P((const r_interval*));
extern r_interval	__ri_sign_f  __P((const r_interval*,const r_interval*));
extern r_interval	__ri_sin_f   __P((const r_interval*));
extern r_interval	__ri_sinh_f  __P((const r_interval*));
extern r_interval 	__ri_sqr_f   __P((const r_interval*));
extern r_interval	__ri_sqrt_f  __P((const r_interval*));
extern r_interval	__ri_tan_f   __P((const r_interval*));
extern r_interval	__ri_tanh_f  __P((const r_interval*));

/* Interval abs */
extern d_interval	__di_fabs_f __P((const d_interval*));
extern r_interval	__ri_fabs_f __P((const r_interval*));
extern q_interval	__qi_fabs_f __P((const q_interval*));

/*
 * Implementation of interval operations & interval specific functions
 */

/* Interval  Addition */
extern r_interval	__ri_add_f __P((const r_interval*,const r_interval*));
extern d_interval	__di_add_f __P((const d_interval*,const d_interval*));
extern q_interval	__qi_add_f __P((const q_interval*,const q_interval*));

/* Interval Subtraction */
extern r_interval	__ri_sub_f __P((const r_interval*,const r_interval*));
extern d_interval	__di_sub_f __P((const d_interval*,const d_interval*));
extern q_interval	__qi_sub_f __P((const q_interval*,const q_interval*));
extern r_interval	__ri_dsub_f __P((const r_interval*,const r_interval*));
extern d_interval	__di_dsub_f __P((const d_interval*,const d_interval*));
extern q_interval	__qi_dsub_f __P((const q_interval*,const q_interval*));

/* Interval Multiplication */
extern r_interval	__ri_mul_f __P((const r_interval*,const r_interval*));
extern d_interval	__di_mul_f __P((const d_interval*,const d_interval*));
extern q_interval	__qi_mul_f __P((const q_interval*,const q_interval*));
extern r_interval	__ri_mul_nornd_f __P((const r_interval*,const r_interval*));
extern d_interval	__di_mul_nornd_f __P((const d_interval*,const d_interval*));
extern q_interval	__qi_mul_nornd_f __P((const q_interval*,const q_interval*));
extern r_interval	__ri_dmul_f __P((const r_interval*,const r_interval*));
extern d_interval	__di_dmul_f __P((const d_interval*,const d_interval*));
extern q_interval	__qi_dmul_f __P((const q_interval*,const q_interval*));

/* Interval Division */
extern r_interval	__ri_div_f __P((const r_interval*,const r_interval*));
extern d_interval	__di_div_f __P((const d_interval*,const d_interval*));
extern q_interval	__qi_div_f __P((const q_interval*,const q_interval*));
extern r_interval	__ri_div_nornd_f __P((const r_interval*,const r_interval*));
extern d_interval	__di_div_nornd_f __P((const d_interval*,const d_interval*));
extern q_interval	__qi_div_nornd_f __P((const q_interval*,const q_interval*));
extern r_interval	__ri_ddiv_f __P((const r_interval*,const r_interval*));
extern d_interval	__di_ddiv_f __P((const d_interval*,const d_interval*));
extern q_interval	__qi_ddiv_f __P((const q_interval*,const q_interval*));
extern r_interval	__ri_divx_f __P((const r_interval*,const r_interval*,const r_interval*));
extern d_interval	__di_divx_f __P((const d_interval*,const d_interval*,const d_interval*));
extern q_interval	__qi_divx_f __P((const q_interval*,const q_interval*,const q_interval*));

/* Interval Negation */
extern r_interval	__ri_neg_f __P((const r_interval*));
extern d_interval	__di_neg_f __P((const d_interval*));
extern q_interval	__qi_neg_f __P((const q_interval*));

/* Interval intersection */
extern r_interval	__ri_ix_f __P((const r_interval*,const r_interval*));
extern d_interval	__di_ix_f __P((const d_interval*,const d_interval*));
extern q_interval	__qi_ix_f __P((const q_interval*,const q_interval*));

/* Interval hull */
extern r_interval	__ri_ih_f __P((const r_interval*,const r_interval*));
extern d_interval	__di_ih_f __P((const d_interval*,const d_interval*));
extern q_interval	__qi_ih_f __P((const q_interval*,const q_interval*));

/* Min, Max */
extern r_interval	__ri_max_f __P((const r_interval*,const r_interval*));
extern d_interval	__di_max_f __P((const d_interval*,const d_interval*));
extern q_interval	__qi_max_f __P((const q_interval*,const q_interval*));
extern r_interval	__ri_min_f __P((const r_interval*,const r_interval*));
extern d_interval	__di_min_f __P((const d_interval*,const d_interval*));
extern q_interval	__qi_min_f __P((const q_interval*,const q_interval*));

/* Aint,   Anint */
extern r_interval	__ri_aint_f 	__P((const r_interval*));
extern d_interval	__di_aint_f 	__P((const d_interval*));
extern q_interval 	__qi_aint_f 	__P((const q_interval*));
extern r_interval	__ri_anint_f 	__P((const r_interval*));
extern d_interval	__di_anint_f 	__P((const d_interval*));
extern q_interval	__qi_anint_f 	__P((const q_interval*));

/* Pow */
extern r_interval	__ri_pow_i_f 	__P((const r_interval*,const int*));
extern r_interval	__ri_pow_xl_f 	__P((const r_interval*,const long long*));
extern d_interval	__di_pow_i_f 	__P((const d_interval*,const int*));
extern d_interval	__di_pow_xl_f 	__P((const d_interval*,const long long*));
extern q_interval	__qi_pow_xl_f 	__P((const q_interval*,const long long*));
extern q_interval	__qi_pow_i_f 	__P((const q_interval*,const int*));
extern r_interval	__ri_pow_f 	__P((const r_interval*,const r_interval*));
extern d_interval	__di_pow_f 	__P((const d_interval*,const d_interval*));
extern q_interval	__qi_pow_f 	__P((const q_interval*,const q_interval*));
extern r_interval	__ri_pow_safe_i_f 	__P((const r_interval*,const d_interval*));
extern r_interval	__ri_pow_safe_xl_f 	__P((const r_interval*,const q_interval*));
extern d_interval	__di_pow_safe_i_f 	__P((const d_interval*,const d_interval*));
extern d_interval	__di_pow_safe_xl_f 	__P((const d_interval*,const q_interval*));
extern q_interval	__qi_pow_safe_i_f 	__P((const q_interval*,const d_interval*));
extern q_interval	__qi_pow_safe_xl_f 	__P((const q_interval*,const q_interval*));

/* Interval constructor */
extern r_interval	__ri_interval_ri_f 	__P((const r_interval*));
extern r_interval	__ri_interval_qi_f 	__P((const q_interval*));
extern r_interval	__ri_interval_di_f 	__P((const d_interval*));
extern d_interval	__di_interval_di_f 	__P((const d_interval*));
extern d_interval	__di_interval_qi_f 	__P((const q_interval*));
extern d_interval	__di_interval_ri_f 	__P((const r_interval*));
extern q_interval	__qi_interval_qi_f 	__P((const q_interval*));
extern q_interval	__qi_interval_di_f 	__P((const d_interval*));
extern q_interval	__qi_interval_ri_f 	__P((const r_interval*));

#ifdef __cplusplus
}
#endif

#endif	/* !defined(_SUNIMATH_H) */
