/*
 * Copyright (c) 2004,2005 by Sun Microsystems, Inc.
 * All rights reserved.
 */

#ifndef __SYS_EMMINTRIN_INTRIN_H
#define __SYS_EMMINTRIN_INTRIN_H

#pragma ident   "@(#)emmintrin.h 1.2 07/01/17"

/*----------------------------------------------------------
      Intrinsics available under SSE2 platform
  ----------------------------------------------------------*/


#ifdef __cplusplus
extern "C" {
#endif

#include <sys/xmmintrin.h>

/*-------------------------------*/
/*   Integer Operations          */
/*-------------------------------*/

/*   Integer Loads/Stores		*/
extern __m128i _mm_loadl_epi64(__m128i const *p);
extern __m128i _mm_load_si128(__m128i const *p);
extern __m128i _mm_loadu_si128(__m128i const *p);
extern void _mm_storel_epi64(__m128i *p1, __m128i p2);
extern void _mm_store_si128(__m128i *p1, __m128i p2);
extern void _mm_storeu_si128(__m128i *p1, __m128i p2);
extern void _mm_maskmoveu_si128(__m128i p1, __m128i p2, char *p3);

/*   Integer Sets			*/
extern __m128i _mm_set1_epi8(char val);
extern __m128i _mm_set1_epi16(short val);
extern __m128i _mm_set1_epi32(int val);
extern __m128i _mm_set1_epi64(__m64 val);
extern __m128i _mm_set_epi8(char b15, char b14, char b13, char b12, 
			    char b11, char b10, char b9,  char b8,  
			    char b7,  char b6,  char b5,  char b4,  
			    char b3,  char b2,  char b1,  char b0);
extern __m128i _mm_set_epi16(short s7, short s6, short s5, short s4,
			     short s3, short s2, short s1, short s0); 
extern __m128i _mm_set_epi32(int i3, int i2, int i1, int i0);
extern __m128i _mm_set_epi64(__m64 q1, __m64 q0);
extern __m128i _mm_setr_epi8(char b0, char b1, char b2, char b3, 
			     char b4, char b5, char b6, char b7,  
			     char b8, char b9, char b10,char b11,  
			     char b12,char b13,char b14,char b15);
extern __m128i _mm_setr_epi16(short s0, short s1, short s2, short s3,
			      short s4, short s5, short s6, short s7); 
extern __m128i _mm_setr_epi32(int i0, int i1, int i2, int i3);
extern __m128i _mm_setr_epi64(__m64 q0, __m64 q1);
extern __m128i _mm_setzero_si128();

/*   Integer Extracts/Inserts		*/
extern int _mm_extract_epi16(__m128i p1, int imm);
extern __m128i _mm_insert_epi16(__m128i p1, int p2, int imm);
extern int _mm_movemask_epi8(__m128i p);

/*   Integer Packs/Unpacks		*/
extern __m128i _mm_packs_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_packs_epi32(__m128i p1, __m128i p2);
extern __m128i _mm_packus_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_shufflehi_epi16(__m128i p1, int imm);
extern __m128i _mm_shufflelo_epi16(__m128i p1, int imm);
extern __m128i _mm_shuffle_epi32(__m128i p1, int imm);
extern __m128i _mm_unpackhi_epi8(__m128i p1, __m128i p2);
extern __m128i _mm_unpackhi_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_unpackhi_epi32(__m128i p1, __m128i p2);
extern __m128i _mm_unpackhi_epi64(__m128i p1, __m128i p2);
extern __m128i _mm_unpacklo_epi8(__m128i p1, __m128i p2);
extern __m128i _mm_unpacklo_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_unpacklo_epi32(__m128i p1, __m128i p2);
extern __m128i _mm_unpacklo_epi64(__m128i p1, __m128i p2);

/*   Integer Arithmetic			*/
extern __m128i _mm_add_epi8(__m128i p1, __m128i p2);
extern __m128i _mm_add_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_add_epi32(__m128i p1, __m128i p2);
extern __m128i _mm_add_epi64(__m128i p1, __m128i p2);
extern __m64   _mm_add_si64(__m64 m1, __m64 m2);
extern __m128i _mm_adds_epi8(__m128i p1, __m128i p2);
extern __m128i _mm_adds_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_adds_epu8(__m128i p1, __m128i p2);
extern __m128i _mm_adds_epu16(__m128i p1, __m128i p2);
extern __m128i _mm_avg_epu8(__m128i p1, __m128i p2);
extern __m128i _mm_avg_epu16(__m128i p1, __m128i p2);
extern __m128i _mm_madd_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_max_epu8(__m128i p1, __m128i p2);
extern __m128i _mm_max_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_min_epu8(__m128i p1, __m128i p2);
extern __m128i _mm_min_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_mulhi_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_mulhi_epu16(__m128i p1, __m128i p2);
extern __m128i _mm_mullo_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_mul_epu32(__m128i p1, __m128i p2);
extern __m64   _mm_mul_su32(__m64 m1, __m64 m2);
extern __m128i _mm_sad_epu8(__m128i p1, __m128i p2);
extern __m128i _mm_sub_epi8(__m128i p1, __m128i p2);
extern __m128i _mm_sub_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_sub_epi32(__m128i p1, __m128i p2);
extern __m128i _mm_sub_epi64(__m128i p1, __m128i p2);
extern __m64   _mm_sub_si64(__m64 m1, __m64 m2);	
extern __m128i _mm_subs_epi8(__m128i p1, __m128i p2);
extern __m128i _mm_subs_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_subs_epu8(__m128i p1, __m128i p2);
extern __m128i _mm_subs_epu16(__m128i p1, __m128i p2);

/*   Integer Logicals			*/
extern __m128i _mm_and_si128(__m128i p1, __m128i p2);
extern __m128i _mm_andnot_si128(__m128i p1, __m128i p2);
extern __m128i _mm_or_si128(__m128i p1, __m128i p2);
extern __m128i _mm_xor_si128(__m128i p1, __m128i p2);

/*   Integer Shifts			*/
extern __m128i _mm_sll_epi16(__m128i p1, __m128i count);
extern __m128i _mm_slli_epi16(__m128i p1, int imm);
extern __m128i _mm_sll_epi32(__m128i p1, __m128i count);
extern __m128i _mm_slli_epi32(__m128i p1, int imm);
extern __m128i _mm_sll_epi64(__m128i p1, __m128i count);
extern __m128i _mm_slli_epi64(__m128i p1, int imm);
extern __m128i _mm_slli_si128(__m128i p1, int imm);
extern __m128i _mm_sra_epi16(__m128i p1, __m128i count);
extern __m128i _mm_srai_epi16(__m128i p1, int imm);
extern __m128i _mm_sra_epi32(__m128i p1, __m128i count);
extern __m128i _mm_srai_epi32(__m128i p1, int imm);
extern __m128i _mm_srl_epi16(__m128i p1, __m128i count);
extern __m128i _mm_srli_epi16(__m128i p1, int imm);
extern __m128i _mm_srl_epi32(__m128i p1, __m128i count);
extern __m128i _mm_srli_epi32(__m128i p1, int imm);
extern __m128i _mm_srl_epi64(__m128i p1, __m128i count);
extern __m128i _mm_srli_epi64(__m128i p1, int imm);
extern __m128i _mm_srli_si128(__m128i p1, int imm);

/*   Integer Comparisons		*/
extern __m128i _mm_cmpeq_epi8(__m128i p1, __m128i p2);
extern __m128i _mm_cmpeq_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_cmpeq_epi32(__m128i p1, __m128i p2);
extern __m128i _mm_cmpgt_epi8(__m128i p1, __m128i p2);
extern __m128i _mm_cmpgt_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_cmpgt_epi32(__m128i p1, __m128i p2);
extern __m128i _mm_cmplt_epi8(__m128i p1, __m128i p2);
extern __m128i _mm_cmplt_epi16(__m128i p1, __m128i p2);
extern __m128i _mm_cmplt_epi32(__m128i p1, __m128i p2);

/*   Integer Conversions		*/
extern __m128i _mm_cvtsi32_si128(int p);
extern int _mm_cvtsi128_si32(__m128i p);

/*   Integer Transfers			*/
extern __m128i _mm_movpi64_epi64(__m64 a);
extern __m64 _mm_movepi64_pi64(__m128i a);
extern __m128i _mm_move_epi64(__m128i a);

/*---------------------------------------------*/
/*   Double Precision Floating Point Operatios */
/*---------------------------------------------*/

/*   Double Precision Loads		*/
extern __m128d _mm_load_sd(const double *p1);		
extern __m128d _mm_load_pd(const double *p1);		
extern __m128d _mm_load1_pd(const double *p1);		
extern __m128d _mm_loadr_pd(const double *p1);		
extern __m128d _mm_loadu_pd(const double *p1);		
extern __m128d _mm_loadh_pd(__m128d p1, const double *p2); 
extern __m128d _mm_loadl_pd(__m128d p1, const double *p2); 

/*   Double Precision Stores		*/
extern void _mm_store_sd(double *p1, __m128d p2);	
extern void _mm_store_pd(double *p1, __m128d p2);	
extern void _mm_store1_pd(double *p1, __m128d p2);	
extern void _mm_storer_pd(double *p1, __m128d p2);	
extern void _mm_storeu_pd(double *p1, __m128d p2);	
extern void _mm_storeh_pd(double *p1, __m128d p2);	
extern void _mm_storel_pd(double *p1, __m128d p2);	

/*   Double Precision Sets		*/
extern __m128d _mm_set_sd(double p1);			
extern __m128d _mm_set_pd(double p1, double p2);	
extern __m128d _mm_set1_pd(double p1);			
extern __m128d _mm_setr_pd(double p1, double p2);	
extern __m128d _mm_setzero_pd(void);			

/*   Double Precision Transfers         */
extern __m128d _mm_move_sd(__m128d p1, __m128d p2);	

/*   Double Precision Arithmetic	*/
extern __m128d _mm_add_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_add_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_sub_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_sub_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_mul_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_mul_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_sqrt_sd(__m128d p1, __m128d p2);
extern __m128d _mm_sqrt_pd(__m128d p1);			
extern __m128d _mm_div_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_div_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_min_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_min_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_max_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_max_pd(__m128d p1, __m128d p2);	

/*   Double Precision Logicals		*/
extern __m128d _mm_and_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_andnot_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_or_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_xor_pd(__m128d p1, __m128d p2);	

/*   Double Precision Comparisons	*/
extern __m128d _mm_cmpeq_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpeq_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmplt_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmplt_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmple_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmple_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpgt_sd(__m128d p1, __m128d p2);
extern __m128d _mm_cmpgt_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpge_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpge_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpneq_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpneq_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpnlt_sd(__m128d p1, __m128d p2);
extern __m128d _mm_cmpnlt_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpnle_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpnle_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpngt_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpngt_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpnge_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpnge_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpord_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpord_sd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpunord_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_cmpunord_sd(__m128d p1, __m128d p2);	
extern int _mm_comieq_sd(__m128d p1, __m128d p2);	
extern int _mm_comilt_sd(__m128d p1, __m128d p2);	
extern int _mm_comile_sd(__m128d p1, __m128d p2);	
extern int _mm_comigt_sd(__m128d p1, __m128d p2);	
extern int _mm_comige_sd(__m128d p1, __m128d p2);	
extern int _mm_comineq_sd(__m128d p1, __m128d p2);	
extern int _mm_ucomieq_sd(__m128d p1, __m128d p2);	
extern int _mm_ucomilt_sd(__m128d p1, __m128d p2);	
extern int _mm_ucomile_sd(__m128d p1, __m128d p2);	
extern int _mm_ucomigt_sd(__m128d p1, __m128d p2);	
extern int _mm_ucomige_sd(__m128d p1, __m128d p2);	
extern int _mm_ucomineq_sd(__m128d p1, __m128d p2);	

/*   Double Precision Conversions	*/
extern __m128 _mm_cvtepi32_ps(__m128i p1);		
extern __m128d _mm_cvtepi32_pd(__m128i p1);		
extern __m128i _mm_cvtps_epi32(__m128 p1);		
extern __m128i _mm_cvttps_epi32(__m128 p1);		
extern __m128i _mm_cvtpd_epi32(__m128d p1);		
extern __m128i _mm_cvttpd_epi32(__m128d p1);		
extern __m128d _mm_cvtps_pd(__m128 p1);			
extern __m128 _mm_cvtpd_ps(__m128d p1);			
extern int _mm_cvtsd_si32(__m128d p1);			
extern int _mm_cvttsd_si32(__m128d p1);			
extern __m128d _mm_cvtsi32_sd(__m128d p1, int p2);	
extern __m128 _mm_cvtsd_ss(__m128 p1, __m128d p2);	
extern __m128d _mm_cvtss_sd(__m128d p1, __m128 p2);	
extern __m64 _mm_cvtpd_pi32(__m128d p1);		
extern __m64 _mm_cvttpd_pi32(__m128d p1);		
extern __m128d _mm_cvtpi32_pd(__m64 p1);		
extern double _mm_cvtsd_f64(__m128d p1);		

/*   Double Precision Miscellaneous	*/
extern __m128d _mm_unpackhi_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_unpacklo_pd(__m128d p1, __m128d p2);	
extern int _mm_movemask_pd(__m128d p1);			
extern __m128d _mm_shuffle_pd(__m128d p1, __m128d p2, int p3); 

/*-------------------------------*/
/*   Cache Operations            */
/*-------------------------------*/

/*   Cache Related			*/
extern void _mm_stream_pd(double *p1, __m128d p2);	
extern void _mm_stream_si128(__m128i *p1, __m128i p2);	
extern void _mm_stream_si32(int *p1, int p2);		
extern void _mm_lfence(void);				
extern void _mm_mfence(void);				
extern void _mm_pause(void);				
extern void _mm_clflush(const void *p1);		

/*-------------------------------*/
/*   Macros                      */
/*-------------------------------*/

#define _MM_SHUFFLE2(x,y) (((x)<<1) | (y))

#ifdef __cplusplus
}
#endif

#endif
