/*
 * Copyright (c) 2004,2005 by Sun Microsystems, Inc.
 * All rights reserved.
 */

#ifndef __SYS_MMINTRIN_INTRIN_H
#define __SYS_MMINTRIN_INTRIN_H

#pragma ident   "@(#)mmintrin.h 1.2 07/01/17"

/*----------------------------------------------------------
      Intrinsics available under MMX platform
  ----------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/sunmedia_types.h>

/*-------------------------------*/
/*   Integer Operations          */
/*-------------------------------*/

/*   Integer Sets			*/
extern __m64 _mm_setzero_si64(void);
extern __m64 _mm_set1_pi8(char c);
extern __m64 _mm_set1_pi16(short s);
extern __m64 _mm_set1_pi32(int i);
extern __m64 _mm_set_pi8(char b7,  char b6,  char b5,  char b4,
                         char b3,  char b2,  char b1,  char b0);
extern __m64 _mm_set_pi16(short s3, short s2, short s1, short s0);
extern __m64 _mm_set_pi32(int i1, int i0);
extern __m64 _mm_setr_pi8(char b0, char b1, char b2, char b3,
                          char b4, char b5, char b6, char b7);
extern __m64 _mm_setr_pi16(short s0, short s1, short s2, short s3);
extern __m64 _mm_setr_pi32(int i0, int i1);

/*   Integer Extracts			*/
extern int _mm_extract_pi16(__m64 a, int i);

/*   Integer Packs/Unpacks		*/
extern __m64 _mm_packs_pi16(__m64 m1, __m64 m2);
extern __m64 _mm_packs_pi32(__m64 m1, __m64 m2);
extern __m64 _mm_packs_pu16(__m64 m1, __m64 m2);
extern __m64 _mm_unpackhi_pi8(__m64 m1, __m64 m2);
extern __m64 _mm_unpackhi_pi16(__m64 m1,__m64 m2);
extern __m64 _mm_unpackhi_pi32(__m64 m1, __m64 m2);
extern __m64 _mm_unpacklo_pi8(__m64 m1, __m64 m2);
extern __m64 _mm_unpacklo_pi16(__m64 m1, __m64 m2);
extern __m64 _mm_unpacklo_pi32(__m64 m1, __m64 m2);

/*   Integer Arithmetic			*/
extern __m64 _mm_add_pi8(__m64 m1, __m64 m2);
extern __m64 _mm_add_pi16(__m64 m1, __m64 m2);
extern __m64 _mm_add_pi32(__m64 m1, __m64 m2);
extern __m64 _mm_adds_pi8(__m64 m1, __m64 m2);
extern __m64 _mm_adds_pi16(__m64 m1, __m64 m2);
extern __m64 _mm_adds_pu8(__m64 m1, __m64 m2);
extern __m64 _mm_adds_pu16(__m64 m1, __m64 m2);
extern __m64 _mm_sub_pi8(__m64 m1, __m64 m2);
extern __m64 _mm_sub_pi16(__m64 m1, __m64 m2);
extern __m64 _mm_sub_pi32(__m64 m1, __m64 m2);
extern __m64 _mm_subs_pi8(__m64 m1, __m64 m2);
extern __m64 _mm_subs_pi16(__m64 m1, __m64 m2);
extern __m64 _mm_subs_pu8(__m64 m1, __m64 m2);
extern __m64 _mm_subs_pu16(__m64 m1, __m64 m2);
extern __m64 _mm_mulhi_pi16(__m64 m1, __m64 m2);
extern __m64 _mm_mullo_pi16(__m64 m1, __m64 m2);
extern __m64 _mm_madd_pi16(__m64 m1, __m64 m2);

/*   Integer Logicals			*/
extern __m64 _mm_and_si64(__m64 m1, __m64 m2);
extern __m64 _mm_andnot_si64(__m64 m1, __m64 m2);
extern __m64 _mm_or_si64(__m64 m1, __m64 m2);
extern __m64 _mm_xor_si64(__m64 m1, __m64 m2);

/*   Integer Shifts			*/
extern __m64 _mm_sll_pi16(__m64 m, __m64 count);
extern __m64 _mm_slli_pi16(__m64 m, int count);
extern __m64 _mm_sll_pi32(__m64 m, __m64 count);
extern __m64 _mm_slli_pi32(__m64 m, int count);
extern __m64 _mm_sll_si64(__m64 m, __m64 count);
extern __m64 _mm_slli_si64(__m64 m, int count);
extern __m64 _mm_sra_pi16(__m64 m, __m64 count);
extern __m64 _mm_srai_pi16(__m64 m, int count);
extern __m64 _mm_sra_pi32(__m64 m, __m64 count);
extern __m64 _mm_srai_pi32(__m64 m, int count);
extern __m64 _mm_srl_pi16(__m64 m, __m64 count);
extern __m64 _mm_srli_pi16(__m64 m, int count);
extern __m64 _mm_srl_pi32(__m64 m, __m64 count);
extern __m64 _mm_srli_pi32(__m64 m, int count);
extern __m64 _mm_srl_si64(__m64 m, __m64 count);
extern __m64 _mm_srli_si64(__m64 m, int count);

/*   Integer Comparisons		*/

extern __m64 _mm_cmpeq_pi8(__m64 m1, __m64 m2);
extern __m64 _mm_cmpeq_pi16(__m64 m1, __m64 m2);
extern __m64 _mm_cmpeq_pi32(__m64 m1, __m64 m2);
extern __m64 _mm_cmpgt_pi8(__m64 m1, __m64 m2);
extern __m64 _mm_cmpgt_pi16(__m64 m1, __m64 m2);
extern __m64 _mm_cmpgt_pi32(__m64 m1, __m64 m2);

/*   Integer Conversions		*/
extern __m64 _mm_cvtsi32_si64(int a);
extern int _mm_cvtsi64_si32(__m64 a);

/*-------------------------------*/
/*   Initialization Operation    */
/*-------------------------------*/

extern void _mm_empty();

/*-------------------------------*/
/*   Macros                      */
/*-------------------------------*/

/* Alternate Names			*/
#define _m_empty	_mm_empty
#define _m_packsswb	_mm_packs_pi16
#define _m_packssdw	_mm_packs_pi32
#define _m_packuswb	_mm_packs_pu16
#define _m_punpckhbw	_mm_unpackhi_pi8
#define _m_punpckhwd	_mm_unpackhi_pi16
#define _m_punpckhdq	_mm_unpackhi_pi32
#define _m_punpcklbw	_mm_unpacklo_pi8
#define _m_punpcklwd	_mm_unpacklo_pi16
#define _m_punpckldq	_mm_unpacklo_pi32
#define _m_paddb	_mm_add_pi8
#define _m_paddw	_mm_add_pi16
#define _m_paddd	_mm_add_pi32
#define _m_paddsb	_mm_adds_pi8
#define _m_paddsw	_mm_adds_pi16
#define _m_paddusb	_mm_adds_pu8
#define _m_paddusw	_mm_adds_pu16
#define _m_psubb	_mm_sub_pi8
#define _m_psubw	_mm_sub_pi16
#define _m_psubd	_mm_sub_pi32
#define _m_psubsb	_mm_subs_pi8
#define _m_psubsw	_mm_subs_pi16
#define _m_psubusb	_mm_subs_pu8
#define _m_psubusw	_mm_subs_pu16
#define _m_pmulhw	_mm_mulhi_pi16
#define _m_pmullw	_mm_mullo_pi16
#define _m_pmaddwd	_mm_madd_pi16
#define _m_pand		_mm_and_si64
#define _m_pandn	_mm_andnot_si64
#define _m_por		_mm_or_si64
#define _m_pxor		_mm_xor_si64
#define _m_psllw	_mm_sll_pi16
#define _m_psllwi	_mm_slli_pi16
#define _m_pslld	_mm_sll_pi32
#define _m_pslldi	_mm_slli_pi32
#define _m_psllq	_mm_sll_si64
#define _m_psllqi	_mm_slli_si64
#define _m_psraw	_mm_sra_pi16
#define _m_psrawi	_mm_srai_pi16
#define _m_psrad	_mm_sra_pi32
#define _m_psradi	_mm_srai_pi32
#define _m_psrlw	_mm_srl_pi16
#define _m_psrlwi	_mm_srli_pi16
#define _m_psrld	_mm_srl_pi32
#define _m_psrldi	_mm_srli_pi32
#define _m_psrlq	_mm_srl_si64
#define _m_psrlqi	_mm_srli_si64
#define _m_pcmpeqb	_mm_cmpeq_pi8
#define _m_pcmpeqw	_mm_cmpeq_pi16
#define _m_pcmpeqd	_mm_cmpeq_pi32
#define _m_pcmpgtb	_mm_cmpgt_pi8
#define _m_pcmpgtw	_mm_cmpgt_pi16
#define _m_pcmpgtd	_mm_cmpgt_pi32
#define _m_from_int	_mm_cvtsi32_si64  
#define _m_to_int	_mm_cvtsi64_si32

#ifdef __cplusplus
}
#endif

#endif
