/*
 * Copyright (c) 2004,2005 by Sun Microsystems, Inc.
 * All rights reserved.
 */

#ifndef __SYS_PMMINTRIN_INTRIN_H
#define __SYS_PMMINTRIN_INTRIN_H

#pragma ident   "@(#)pmmintrin.h 1.2 07/01/17"

/*----------------------------------------------------------
      Intrinsics available under SSE3 platform
  ----------------------------------------------------------*/


#ifdef __cplusplus
extern "C" {
#endif

#include <sys/emmintrin.h>

/*-------------------------------*/
/*   Integer Operation           */
/*-------------------------------*/

/*   Integer Unaligned Load		*/
extern __m128i _mm_lddqu_si128(const __m128i *p);	

/*----------------------------------------------*/
/*   Single Precision Floating Point Operations */
/*----------------------------------------------*/

/*   Single Precision Vectors		*/
extern __m128 _mm_movehdup_ps(__m128 p);		
extern __m128 _mm_moveldup_ps(__m128 p);		
extern __m128 _mm_addsub_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_hadd_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_hsub_ps(__m128 p1, __m128 p2);	

/*----------------------------------------------*/
/*   Double Precision Floating Point Operatios  */
/*----------------------------------------------*/

/*   Double Precision Vectors		*/
extern __m128d _mm_loaddup_pd(const double *p);		
extern __m128d _mm_movedup_pd(__m128d p);		
extern __m128d _mm_addsub_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_hadd_pd(__m128d p1, __m128d p2);	
extern __m128d _mm_hsub_pd(__m128d p1, __m128d p2);	

/*-------------------------------*/
/*   Miscellaneous Operations    */
/*-------------------------------*/

/*   sse3 new instructions		*/
extern void _mm_monitor(const void *p1, unsigned p2, unsigned p3);
extern void _mm_mwait(unsigned p1, unsigned p2);

/*-------------------------------*/
/*   Macros                      */
/*-------------------------------*/

/* More bits for MXCSR.  */
#define _MM_DENORMALS_ZERO_MASK		0x0040
#define _MM_DENORMALS_ZERO_ON		0x0040
#define _MM_DENORMALS_ZERO_OFF		0x0000

#define _MM_GET_DENORMALS_ZERO_MODE() \
	(_mm_getcsr() & _MM_DENORMALS_ZERO_MASK)
#define _MM_SET_DENORMALS_ZERO_MODE(mode) \
	_mm_setcsr ((_mm_getcsr() & ~_MM_DENORMALS_ZERO_MASK) \
		    | (mode))

#ifdef __cplusplus
}
#endif

#endif
