/*
 * Copyright (c) 2004,2005 by Sun Microsystems, Inc.
 * All rights reserved.
 */

#ifndef __SYS_XMMINTRIN_INTRIN_H
#define __SYS_XMMINTRIN_INTRIN_H

#pragma ident   "@(#)xmmintrin.h 1.2 07/01/17"

/*----------------------------------------------------------
      Intrinsics available under SSE platform
  ----------------------------------------------------------*/


#ifdef __cplusplus
extern "C" {
#endif

#include <sys/mmintrin.h>

/*-------------------------------*/
/*   Integer Operations          */
/*-------------------------------*/

/*   Integer Stores			*/
extern void _mm_maskmove_si64(__m64 p1, __m64 p2, char *p3);

/*   Integer Inserts			*/
extern __m64 _mm_insert_pi16(__m64 p1, int p2, int imm);

/*   Integer Arithmetic			*/
extern __m64 _mm_sad_pu8(__m64 p1, __m64 p2);
extern __m64 _mm_mulhi_pu16(__m64 p1, __m64 p2);
extern __m64 _mm_avg_pu8(__m64 p1, __m64 p2);
extern __m64 _mm_avg_pu16(__m64 p1, __m64 p2);
extern __m64 _mm_max_pi16(__m64 p1, __m64 p2);
extern __m64 _mm_max_pu8(__m64 p1, __m64 p2);
extern __m64 _mm_min_pi16(__m64 p1, __m64 p2);
extern __m64 _mm_min_pu8(__m64 p1, __m64 p2);
extern int _mm_movemask_pi8(__m64 p);

/*   Integer Miscellaneous		*/
extern __m64 _mm_shuffle_pi16(__m64 p1, int imm);

/*------------------------------------------------*/
/*   Single Precision Floating Point Operations   */
/*------------------------------------------------*/

/*   Single Precision Loads		*/
extern __m128 _mm_load_ss(const float *p);		
extern __m128 _mm_load_ps(const float *p);		
extern __m128 _mm_load1_ps(const float *p);		
extern __m128 _mm_loadr_ps(const float *p);		
extern __m128 _mm_loadu_ps(const float *p);		
extern __m128 _mm_loadh_pi(__m128 p1, const __m64 *p2);	
extern __m128 _mm_loadl_pi(__m128 p1, const __m64 *p2); 

/*   Single Precision Stores		*/
extern void _mm_store_ss(float *p1, __m128 p2);		
extern void _mm_store_ps(float *p1, __m128 p2);		
extern void _mm_store1_ps(float *p1, __m128 p2);	
extern void _mm_storer_ps(float *p1, __m128 p2);	
extern void _mm_storeu_ps(float *p1, __m128 p2);	
extern void _mm_storeh_pi(__m64 *p1, __m128 p2);	
extern void _mm_storel_pi(__m64 *p1, __m128 p2);	

/*   Single Precision Sets 		*/
extern __m128 _mm_set_ss(float p);				   
extern __m128 _mm_set_ps(float p1, float p2, float p3, float p4);  
extern __m128 _mm_set1_ps(float p);				   
extern __m128 _mm_setr_ps(float p1, float p2, float p3, float p4); 
extern __m128 _mm_setzero_ps(void);				   

/*   Single Precision Transfers		*/
extern __m128 _mm_move_ss(__m128 p1, __m128 p2);	

/*   Single Precision Arithmetic	*/
extern __m128 _mm_add_ss(__m128 p1, __m128 p2);		
extern __m128 _mm_add_ps(__m128 p1, __m128 p2);		
extern __m128 _mm_sub_ss(__m128 p1, __m128 p2);		
extern __m128 _mm_sub_ps(__m128 p1, __m128 p2);		
extern __m128 _mm_mul_ss(__m128 p1, __m128 p2);		
extern __m128 _mm_mul_ps(__m128 p1, __m128 p2);		
extern __m128 _mm_div_ss(__m128 p1, __m128 p2);		
extern __m128 _mm_div_ps(__m128 p1, __m128 p2);		
extern __m128 _mm_sqrt_ss(__m128 p);			
extern __m128 _mm_sqrt_ps(__m128 p);			
extern __m128 _mm_rcp_ss(__m128 p);			
extern __m128 _mm_rcp_ps(__m128 p);			
extern __m128 _mm_rsqrt_ss(__m128 p);			
extern __m128 _mm_rsqrt_ps(__m128 p);			
extern __m128 _mm_min_ss(__m128 p1, __m128 p2);		
extern __m128 _mm_min_ps(__m128 p1, __m128 p2);		
extern __m128 _mm_max_ss(__m128 p1, __m128 p2);		
extern __m128 _mm_max_ps(__m128 p1, __m128 p2);		

/*   Single Precision Logicals		*/
extern __m128 _mm_and_ps(__m128 p1, __m128 p2);		
extern __m128 _mm_andnot_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_or_ps(__m128 p1, __m128 p2);		
extern __m128 _mm_xor_ps(__m128 p1, __m128 p2);		

/*   Single Precision Comparisons	*/
extern __m128 _mm_cmpeq_ss(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpeq_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_cmplt_ss(__m128 p1, __m128 p2);	
extern __m128 _mm_cmplt_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_cmple_ss(__m128 p1, __m128 p2);	
extern __m128 _mm_cmple_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpgt_ss(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpgt_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpge_ss(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpge_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpneq_ss(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpneq_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpnlt_ss(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpnlt_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpnle_ss(__m128 p1, __m128 p2);	 
extern __m128 _mm_cmpnle_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpngt_ss(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpngt_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpnge_ss(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpnge_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpord_ss(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpord_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpunord_ss(__m128 p1, __m128 p2);	
extern __m128 _mm_cmpunord_ps(__m128 p1, __m128 p2);	
extern int _mm_comieq_ss(__m128 p1, __m128 p2);		
extern int _mm_comilt_ss(__m128 p1, __m128 p2);		
extern int _mm_comile_ss(__m128 p1, __m128 p2);		
extern int _mm_comigt_ss(__m128 p1, __m128 p2);		
extern int _mm_comige_ss(__m128 p1, __m128 p2);		
extern int _mm_comineq_ss(__m128 p1, __m128 p2);	
extern int _mm_ucomieq_ss(__m128 p1, __m128 p2);	
extern int _mm_ucomilt_ss(__m128 p1, __m128 p2);	
extern int _mm_ucomile_ss(__m128 p1, __m128 p2);	
extern int _mm_ucomigt_ss(__m128 p1, __m128 p2);	
extern int _mm_ucomige_ss(__m128 p1, __m128 p2);	
extern int _mm_ucomineq_ss(__m128 p1, __m128 p2);	

/*   Single Precision Conversions	*/
extern int _mm_cvtss_si32(__m128 p);			
extern int _mm_cvttss_si32(__m128 p);			
extern __m128 _mm_cvtsi32_ss(__m128 p1, int p2);	
extern __m64 _mm_cvtps_pi32(__m128 p);			
extern __m64 _mm_cvttps_pi32(__m128 p);			
extern __m128 _mm_cvtpi32_ps(__m128 p1, __m64 p2);	
extern float _mm_cvtss_f32(__m128 p);			

/*   Single Precision Miscellaneous	*/
extern __m128 _mm_unpackhi_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_unpacklo_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_movehl_ps(__m128 p1, __m128 p2);	
extern __m128 _mm_movelh_ps(__m128 p1, __m128 p2);	
extern int _mm_movemask_ps(__m128 p);			
extern __m128 _mm_shuffle_ps(__m128 p1, __m128 p2, unsigned int imm8); 

/*-------------------------------*/
/*   Cache Operations            */
/*-------------------------------*/

/*   Cache Related			*/
extern void _mm_stream_pi(__m64 *p1, __m64 p2);		
extern void _mm_stream_ps(float *p1, __m128 p2);	
extern void _mm_sfence(void);
extern void _mm_prefetch(const char *p1, int p2);	
extern unsigned int _mm_getcsr(void);			
extern void _mm_setcsr(unsigned int p);			

/*-------------------------------*/
/*   Macros                      */
/*-------------------------------*/

#define _MM_SHUFFLE(fp3,fp2,fp1,fp0) \
       (((fp3) << 6) | ((fp2) << 4) | ((fp1) << 2) | (fp0))

/*   Constants for _mm_prefetch		*/
#define _MM_HINT_NTA		0
#define _MM_HINT_T0		1
#define _MM_HINT_T1		2
#define _MM_HINT_T2		3

/*   Bits in MXCSR 		        */
#define _MM_EXCEPT_MASK		0x003f
#define _MM_EXCEPT_INVALID	0x0001
#define _MM_EXCEPT_DENORM	0x0002
#define _MM_EXCEPT_DIV_ZERO	0x0004
#define _MM_EXCEPT_OVERFLOW	0x0008
#define _MM_EXCEPT_UNDERFLOW	0x0010
#define _MM_EXCEPT_INEXACT	0x0020

#define _MM_MASK_MASK		0x1f80
#define _MM_MASK_INVALID	0x0080
#define _MM_MASK_DENORM		0x0100
#define _MM_MASK_DIV_ZERO	0x0200
#define _MM_MASK_OVERFLOW	0x0400
#define _MM_MASK_UNDERFLOW	0x0800
#define _MM_MASK_INEXACT	0x1000

#define _MM_ROUND_MASK		0x6000
#define _MM_ROUND_NEAREST	0x0000
#define _MM_ROUND_DOWN		0x2000
#define _MM_ROUND_UP		0x4000
#define _MM_ROUND_TOWARD_ZERO	0x6000

#define _MM_FLUSH_ZERO_MASK	0x8000
#define _MM_FLUSH_ZERO_ON	0x8000
#define _MM_FLUSH_ZERO_OFF	0x0000

/*   Alternate Name Macros		*/
#define _m_pextrw		_mm_extract_pi16
#define _m_maskmovq		_mm_maskmove_si64
#define _m_pshufw		_mm_shuffle_pi16
#define _m_pinsrw		_mm_insert_pi16
#define _m_psadbw		_mm_sad_pu8
#define _m_pmulhuw		_mm_mulhi_pu16
#define _m_pavgb		_mm_avg_pu8
#define _m_pavgw		_mm_avg_pu16
#define _m_pmaxsw		_mm_max_pi16
#define _m_pmaxub		_mm_max_pu8
#define _m_pminsw		_mm_min_pi16
#define _m_pminub		_mm_min_pu8
#define _m_pmovmskb		_mm_movemask_pi8
#define _mm_set_ps1		_mm_set1_ps
#define _mm_load_ps1		_mm_load1_ps
#define _mm_store_ps1		_mm_store1_ps
#define _mm_cvt_ss2si		_mm_cvtss_si32
#define _mm_cvt_ps2pi		_mm_cvtps_pi32
#define _mm_cvtt_ss2si		_mm_cvttss_si32	
#define _mm_cvtt_ps2pi		_mm_cvttps_pi32	
#define _mm_cvt_si2ss		_mm_cvtsi32_ss
#define _mm_cvt_pi2ps		_mm_cvtpi32_ps			


#ifdef __cplusplus
}
#endif

#endif
